/**
* This is the main source file for spudslices, the library that lets you make
* shapes.
*/
/**
* The main namespace for spudslices, the library that lets you make shapes.
*/
declare namespace spudslices {
    /**
    * A string that identifies the version number.
    */
    const version: string;
    /**
    * Find the distance of (x,y) from the origin (0,0)
    */
    const distance: (x: number, y: number) => number;
    /**
    * Find the rotation (in radians) of (x,y) from (0,1) around the origin
    * (0,0)
    */
    function findRot(x: number, y: number): number;
    /**
    * Rotate a point (x,y) around (0,0)
    * @param rad The rotation to offset the new rotation from (0,distance),
    * where distance is the distance of (x,y) from (0,0)
    */
    function rawRotate(x: number, y: number, rad: number): [number, number];
    /**
    * Get the location of where a point will be rotated (given `rad`) from it's
    * current location (x,y)
    */
    const rotate: (x: number, y: number, rad: number) => [number, number];
    /**
    * A database of the functions that get called when checking to see if any
    * two shapes are touching.
    *
    * @see [[Shape.collisionWith]] for more info.
    */
    let collisionDetectors: {
        [index: string]: {
            [index: string]: (sh: Shape) => any[];
        };
    };
    /**
    * The base class for all shapes.
    */
    class Shape {
        [x: string]: any;
        /**
        * This string is what identifies to [[Shape.collisionWith]] which of the
        * [[spudslices.collisionDetectors]] to use.
        */
        category: string;
        /**
        * This identifies to internal mechanisims that this is a 2d object.
        *
        * These internal mechanisims will expect the length of each
        * array inside of `[[Shape.points]] to equal this value.
        *
        * Sometime in the future, if 3D support is ever added, this will be
        * used to have any-dimention shapes, but it will require a few more
        * base shapes, and much, much more complicated collisions.
        */
        dimensions: number;
        /**
        * Even though (at time of writing) this is type `any[]` it is one of
        *  two things:
        *
        *  * `number[][]`, given that it is an ordinary point.
        *     (See [[dimensions]]) for more info.
        *  * `number[]`, given that it is the first point, and the class type
        *    is `Circle` (or one of it's children). In this case, this point is
        *    the radius, and the next point is the center of the circle.
        *
        * This will be changed in v2
        */
        points: any[];
        /**
        * The color that points are rendered when calling either [[drawOn]] or
        * [[drawPointsOn]]. Overrided by [[pointColors]].
        */
        pointColor: string;
        /**
        * The color that each point is rendered when calling either [[drawOn]] or
        * [[drawPointsOn]]. Defaults to [[pointColor]].
        */
        pointColors: string[];
        /**
        * The radius that points are rendered when calling either [[drawOn]] or
        * [[drawPointsOn]].
        */
        pointSize: number;
        /**
        * Each item in the array symbolises a segment, where each item within
        * is a number referring to the index of its respective point. The
        * length of the innermost array should always be `2` as segments are
        * comprised of exactly two points.
        */
        segments: [number, number][];
        /**
        * The color that segments are rendered when calling either [[drawOn]] or
        * [[drawSegmentsOn]]. Overrided by [[segmentColors]].
        */
        segmentColor: string;
        /**
        * The color that each segment is rendered when calling either [[drawOn]] or
        * [[drawSegmentsOn]]. Defaults to [[segmentColor]].
        */
        segmentColors: string[];
        /**
        * The with that segments are rendered when calling either [[drawOn]] or
        * [[drawPointsOn]].
        */
        segmentSize: number;
        /**
        * Each item in the array symbolises a face, where each item within is a
        * number referring to the index of its respective segment.
        */
        faces: number[][];
        /**
        * The color that faces are rendered when calling either [[drawOn]] or
        * [[drawFacesOn]]. Overrided by [[faceColors]].
        */
        faceColor: string;
        /**
        * The color that each face is rendered when calling either [[drawOn]] or
        * [[drawFacesOn]]. Defaults to [[faceColor]].
        */
        faceColors: string[];
        /**
        * Draws this [[Shape]] instance on this CanvasRenderingContext2D
        * instance.
        */
        drawOn(ctx: CanvasRenderingContext2D): Shape;
        /**
        * Draws the points of this [[Shape]] instance on this
        * CanvasRenderingContext2D instance.
        */
        drawPointsOn(ctx: CanvasRenderingContext2D): Shape;
        /**
        * Draws the segments of this [[Shape]] instance on this
        * CanvasRenderingContext2D instance.
        */
        drawSegmentsOn(ctx: CanvasRenderingContext2D): Shape;
        /**
        * Draws the faces of this [[Shape]] instance on this
        * CanvasRenderingContext2D instance.
        */
        drawFacesOn(ctx: CanvasRenderingContext2D): Shape;
        /**
        * Transposes the shape by (x,y)
        */
        transpose(x: number, y: number): Shape;
        /**
        * A basic scale function.
        *
        * > NOTE: Doesnt morph the shape of [[Circle]] elements, but it does
        * > scale its position, and will take the average of x and y to figure
        * > out how much to scale the radius.
        * >
        * > TL;DR: Circles scaled with this function still are circles, never
        * > ovals.
        *
        * @param x How much the shape is scaled along the x-position
        * @param y Inferred to be `x` if missing. If not missing, how
        *   much the shape is scaled along the y-position.
        */
        scale(x: number, y?: number): Shape;
        /**
        * Rotate the shape around (x,y)
        * @param rad How many radians to rotate it by.
        */
        rotate(x: number, y: number, rad: number): Shape;
        /**
        * Round all points in the function. (Will also round the radius of a circle)
        * @param otherF Optional function to use instead of `Math.round`
        */
        roundPoints(otherF?: {
            (x: number): number;
        }): Shape;
        /**
        * Rotate the shape around its own center.
        * @param rad How far to rotate.
        */
        rotCenter(rad: number): Shape;
        /**
        * Return a duplicate of this object, removing all pointers to the
        * original.
        */
        makeDup(): any;
        /**
        * Search for the [[Shape.category]] of both, then run the
        * this one's function if it can be found. If not, run this the other
        * ones function.
        *
        * > NOTE: I plan on having it call the other ones first, but there is
        * > a deep-rooted bug that doesn't properly call the other's.
        * >
        * > The
        * > reasoning behind having the other's called first is the fact that
        * > it allows for someone to create their own collisionDetector &/or
        * > type of shape in a better way.
        */
        collisionWith(sh: Shape): any[];
        /**
        * Find the center of any shape, relative to the points.
        */
        findCenter(): any[];
    }
    /**
    * The base class for circle- like shapes.
    *
    * > Note: Causes minor conflict as [[Shape.points]] [0] is the radius,
    * > whereas the rest of the library expects the type of [[Shape.points]] to
    * > be `number[][]`. This may be redesigned in a future major release.
    * >
    * > (remove in v2)
    */
    class Circle extends Shape {
        /**
        * Constuctor for [[Circle]]
        * @param x X-location of Circle
        * @param y Y-location of Circle
        * @param r Radius of [[Circle]]
        */
        constructor(x: number, y: number, r: number);
        /**
        * Find the center of a [[Circle]].
        */
        findCenter: () => [number, number];
    }
    class Polygon extends Shape {
        /**
        * @param args Each point of the polygon. Points are pre-connected with
        * segments, and the last point is automatically connected to the first
        * point.
        */
        constructor(...args: number[][] | number[]);
        /**
        * Makes a point at the center, draws lines from that point to every
        * other point, and ends up cutting up the shape like a pizza.
        *
        * > NOTE: Assumes the shape to have 1 face.
        */
        convertToTriangles(): Polygon[];
        /**
        * Removes the segments' shared point, then replaces them with a single
        * segment.
        * @param segA Index of the first segment
        * @param segB Index of the second segement
        */
        joinSegments(segA: number, segB: number): Shape;
        /**
        * Replaces this segment with two segments, placing a point in the
        * middle, unless overridden.
        * @param segNum Index of segment to be replaced
        * @param sugP Optional location of new point.
        * @param color Not implemented yet, possible way to define the color of
        * the new point, and the colors of the new segments.
        */
        splitSegment(segNum: number, sugP?: number[]): Shape;
    }
    /**
    * A right triangle.
    *
    * The 90 degree angle is at (x,y).
    * Next point is at (x+w,y), then (x,y+h)
    */
    class RightTriangle extends Polygon {
        constructor(x: number, y: number, w: number, h: number);
    }
    /**
    * A 45,45,90 triangle.
    */
    class IsosolesRightTriangle extends RightTriangle {
        /**
        * @param w Both the width and height of the Right triangle.
        */
        constructor(x: number, y: number, w: number);
    }
    class Rectagle extends Polygon {
        constructor(x: number, y: number, w: number, h: number);
    }
    class Square extends Rectagle {
        /**
        * @param w Both the width and height of the square.
        */
        constructor(x: number, y: number, w: number);
    }
    /**
    * A way of defining a shape, such as an Octogon.
    *
    * > I don't plan on adding Octogon, Pentagon..., as this is so easy to use.
    */
    class EqualDistShape extends Polygon {
        /**
        * Make an [[EqualDistShape]], with one point on the right side, due to
        * the way that [[spudslices.rawRotate]] works.
        * @param sideCount Number of sides.
        * @param radius Distance of all points from (x,y)
        */
        constructor(x: number, y: number, sideCount: number, radius: number);
    }
    /**
    * > **Will be DEPRECATED in v2**
    *
    * Include ss and spudslices in the main export
    *
    * Put here for compatibility with v1.2.3. Please don't use in new code.
    * (If you would like me to keep these redundant exports, tell me, and I
    * will consider canceling their deprecation)
    *
    * If you are using `import {ss} from 'spudslices'` or
    * `import {spudslices} from 'spudslices'` use `import ss from 'spudslices`
    * or `import spudslices from 'spudslices'` instead.
    */
    let ss: any, spudslices: any;
}
export = spudslices;
